from .general_tags import *
from .a_tags import *
from .b_tags import *
from .c_tags import *
from .d_tags import *
from .e_tags import *
from .f_tags import *
from .h_tags import *
from .i_tags import *
from .k_tags import *
from .l_tags import *
from .m_tags import *
from .n_tags import *
from .o_tags import *
from .p_tags import *
from .q_tags import *
from .r_tags import *
from .s_tags import *
from .t_tags import *
from .u_tags import *
from .v_tags import *
from .w_tags import *
